#!/bin/bash

CONFVARS="lockfile datadir logdir configdir suffix ip_start ip_end ip_mask port_begin port_num protos fw_ports fw_file if_name if_ip local_if local_ip local_dns"
GENVARS="proto protox port"

saveovpn()
{
cat > "$FNAME" << EOF
local %local_ip%
port %port%
proto %protox%
dev tap%port%

persist-tun
persist-key

ca %configdir%/%suffix%/ca.crt
cert %configdir%/%suffix%/cert.crt
key %configdir%/%suffix%/cert.key
dh %configdir%/%suffix%/dh2048.pem

mode server
push "route-gateway %if_ip%"

;client-to-client
duplicate-cn

username-as-common-name
auth-user-pass-optional

user root

tls-server
keepalive 10 30
cipher AES-256-CBC

status %datadir%/%port%-%proto%.status
status-version 2

log-append %logdir%/%port%-%proto%-%suffix%.log

mute-replay-warnings
mute 3

script-security 3
up "/usr/local/bin/rapidvpn-script.sh"
down-pre "/usr/local/bin/rapidvpn-script.sh"
client-connect "/usr/local/bin/rapidvpn-script.sh"
client-disconnect "/usr/local/bin/rapidvpn-script.sh"
auth-user-pass-verify "/usr/local/bin/rapidvpn-script.sh" via-env
EOF

  for Vname in $CONFVARS $GENVARS; do
    eval "Vval=\$${Vname}"
    Vval=${Vval//\//\\/}
    sed -i "s/%${Vname}%/${Vval}/g" "$FNAME"
  done

  echo "" >> "$FNAME"
  for Vname in $CONFVARS $GENVARS; do
    eval "Vval=\$${Vname}"
    echo "setenv-safe $Vname \"$Vval\"" >> "$FNAME"
  done
}

myexit()
{
  if [ -z "$1" ]; then exit 1; fi
  if [ -n "$2" ]; then echo "ERROR: $2"; fi
  exit $1
}

check_ip()
{
  if [ -z "$1" ] || [[ ! $1 =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then myexit 1 "malformed IP"; fi
  read p1 p2 p3 p4 <<<$(IFS='.'; echo $1)
  if [ "$p1" -lt "0" ] || [ "$p1" -gt "255" ]; then myexit 1 "malformed IP '$1'"; fi
  if [ "$p2" -lt "0" ] || [ "$p2" -gt "255" ]; then myexit 1 "malformed IP '$1'"; fi
  if [ "$p3" -lt "0" ] || [ "$p3" -gt "255" ]; then myexit 1 "malformed IP '$1'"; fi
  if [ "$p4" -lt "0" ] || [ "$p4" -gt "255" ]; then myexit 1 "malformed IP '$1'"; fi
}

usage()
{
  echo "Usage: $0 COMMAND CONFIG"
  echo "COMMAND:"
  echo "  install - creates openvpn config files"
  echo "  firewall - generates firewall rules for load balancing"
  echo "CONFIG:"
  echo "  Path to .rapid config file"
  exit 1
}

readconfig()
{
  . "$1"
  for X in $CONFVARS; do
    eval "V=\$${X}"
    if [ -z "$V" ]; then myexit 1 "Missing variable '$X'"; fi
    if [ "${X:(-3)}" == "dir" ]; then
      if [ ! -d "$V" ]; then mkdir -p "$V" >/dev/null 2>/dev/null; fi
      if [ ! -d "$V" ]; then myexit 1 "Unable to create directory '$V'"; fi
    fi
    if [ "${X:(-3)}" == "_ip" ] || [ "${X:0:3}" == "ip_" ] ; then
      check_ip "$V"
    fi
  done
}

if [ -z "$1" ] || [ -z "$2" ] || [ ! -f "$2" ]; then usage; fi
CMD="$1"
readconfig "$2"

if [ "$CMD" == "install" ]; then

  port="$port_begin"
  for proto in $protos; do
    if [ "$proto" == "udp" ]; then protox="udp"
      else if [ "$proto" == "tcp" ]; then protox="tcp-server"
        else myexit 1 "Proto must be 'tcp' or 'udp'"; fi
    fi
    eval "port_$proto=\$port"

    for I in $(seq 1 $port_num); do
      FNAME="${configdir}/${port}-${proto}-${suffix}.conf"
      echo "$FNAME"
      saveovpn
      let "port=port+1"
    done

  done

  exit 0
fi

if [ "$CMD" == "firewall" ]; then
  FNAME="$fw_file"
  echo "#!/bin/bash" > "$FNAME"
  echo "# Generated on $(date)" >> "$FNAME"

  port="$port_begin"
  for proto in $protos; do
    if [ "$proto" != "udp" ] && [ "$proto" != "tcp" ]; then myexit 1 "Proto must be 'tcp' or 'udp'"; fi
    echo "" >> "$FNAME"
    echo "# INPUT $proto" >> "$FNAME"
    let "out_port=port+port_num-1"
    echo "\$IPTABLES -A INPUT -p $proto -i $local_if -d $local_ip --dport ${port}:${out_port} -m state --state NEW -j ACCEPT" >> "$FNAME"

    echo "" >> "$FNAME"
    echo "# DNAT $proto" >> "$FNAME"
    for X in $fw_ports; do
      read in_port in_proto <<<$(IFS='/'; echo $X)
      if [ "$in_proto" == "$proto" ] && [[ $in_port =~ ^[0-9]+$ ]]; then
        echo "# ${in_port}/${in_proto}" >> "$FNAME"
        for I in $(seq 1 $port_num); do
          let "out_port=port+I-1"
          if [ "$I" -gt "1" ]; then
            PROB=$(awk "BEGIN{print 1.0 / $I}")
            echo "\$IPTABLES -t nat -I PREROUTING -i $local_if -d $local_ip -p $proto --dport $in_port -m state --state NEW -m statistic --mode random --probability $PROB -j DNAT --to ${local_ip}:${out_port}" >> "$FNAME"
          else
            echo "\$IPTABLES -t nat -I PREROUTING -i $local_if -d $local_ip -p $proto --dport $in_port -m state --state NEW -j DNAT --to ${local_ip}:${out_port}" >> "$FNAME"
          fi
        done
      fi
    done

    let "port=port+port_num"
  done

  exit 0
fi

usage
