#!/bin/bash

run_trap()
{
  ret="$?"
  rm -f "$LOCKFILE"
  exit $ret
}

myexit()
{
  if [ -z "$1" ]; then exit 1; fi
  if [ -n "$2" ]; then echo "ERROR: $2"; fi
  exit $1
}

check_ip()
{
  if [ -z "$1" ] || [[ ! $1 =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then myexit 1 "malformed IP"; fi
  read p1 p2 p3 p4 <<<$(IFS='.'; echo $1)
  if [ "$p1" -lt "0" ] || [ "$p1" -gt "255" ]; then myexit 1 "malformed IP"; fi
  if [ "$p2" -lt "0" ] || [ "$p2" -gt "255" ]; then myexit 1 "malformed IP"; fi
  if [ "$p3" -lt "0" ] || [ "$p3" -gt "255" ]; then myexit 1 "malformed IP"; fi
  if [ "$p4" -lt "0" ] || [ "$p4" -gt "255" ]; then myexit 1 "malformed IP"; fi
}

next_ip()
{
  read p1 p2 p3 p4 <<<$(IFS='.'; echo $1)

  let "p4=p4+1"
  if [ "$p4" -gt "255" ]; then let "p4=0";let "p3=p3+1"
    if [ "$p3" -gt "255" ]; then let "p3=0";let "p2=p2+1"
      if [ "$p2" -gt "255" ]; then let "p2=0";let "p1=p1+1"
        if [ "$p1" -gt "255" ]; then myexit 1 "out of IP addresses"; fi
      fi
    fi
  fi

  IP="${p1}.${p2}.${p3}.${p4}"
  if [ "$IP" == "$IP_END" ]; then
    IP="$IP_START"
  fi
}

check_if_used()
{
  rIP=${1//./\\.}
  for F in $(ls ${DATADIR}/*.status); do
    D=$(grep -E "^CLIENT_LIST,[-a-zA-Z0-9\._:]+,[-a-zA-Z0-9\._:]+,$rIP(,.*)?\$" $F | wc -l)
    if [ "$D" -gt "0" ]; then return 1; fi
  done
  return 0
}

find_next_unused_ip()
{
  if [ ! -f "$LASTIP_FNAME" ]; then
    IP="$IP_START"
  else
    IP=$(head -n1 "$LASTIP_FNAME")
    check_ip "$IP"
  fi

  let "OK=128"
  while [ "$OK" -gt "0" ]; do
    next_ip "$IP"
    if check_if_used "$IP"; then
      let "OK=-1"
    else
      let "OK=OK-1"
    fi
  done
  if [ "$OK" -eq "0" ]; then myexit 1 "could not find any free IP address"; fi
}

find_userpass_ip()
{
  if [ ! -f "$CLIENTS_FNAME" ]; then return 1; fi
  read r_pass r_ip <<<$(grep "^CLIENT_LIST,$username," $CLIENTS_FNAME | head -n1 | awk -F',' '{print $3" "$4}')
  if [ -z "$r_pass" ] && [ -z "$r_ip" ]; then return 1; fi

  if [ "$1" == "checkpass" ] && [ "$r_pass" != "$password" ]; then myexit 1 "wrong password"; fi
  check_ip "$r_ip"
  IP="$r_ip"
  return 0
}

if [ -z "$dev" ] || [ -z "$script_type" ] || [ -z "$OPENVPN_lockfile" ] || [ -z "$OPENVPN_datadir" ] || [ -z "$OPENVPN_ip_start" ] || [ -z "$OPENVPN_ip_end" ] || [ -z "$OPENVPN_ip_mask" ] || [ -z "$OPENVPN_if_name" ] || [ -z "$OPENVPN_if_ip" ]; then
  echo "Same basic environment variables are missing!"
  exit 2
fi

LOCKFILE="$OPENVPN_lockfile"
DATADIR="$OPENVPN_datadir"
IP_START="$OPENVPN_ip_start"
IP_END="$OPENVPN_ip_end"
IP_MASK="$OPENVPN_ip_mask"
IF_NAME="$OPENVPN_if_name"
IF_IP="$OPENVPN_if_ip"

if [ ! -d "$DATADIR" ]; then
  echo "Data directory '$DATADIR' not found!"
  exit 3
fi

LASTIP_FNAME="${DATADIR}/lastip.txt"
CLIENTS_FNAME="${DATADIR}/clients.status"

check_ip "$IP_START"
check_ip "$IP_END"
check_ip "$IP_MASK"
check_ip "$IF_IP"

let "OK=2"
while [ "$OK" -gt "0" ]; do
  if ( set -o noclobber; echo "$$" > "$LOCKFILE") 2> /dev/null; then
    trap 'run_trap' INT TERM EXIT
    let "OK=-1"
  else
    let "OK=OK-1"
    sleep 1
  fi
done
if [ "$OK" -eq "0" ]; then myexit 1 "Could not acquire lock!"; fi

if [ "$script_type" == "up" ]; then

  /sbin/ifconfig $IF_NAME >/dev/null 2>/dev/null
  if [ "$?" -gt "0" ]; then
    /usr/sbin/brctl addbr $IF_NAME
    /sbin/ifconfig $IF_NAME $IF_IP netmask $IP_MASK
  fi

  /sbin/ifconfig $dev up
  /usr/sbin/brctl addif $IF_NAME $dev

  exit 0
fi

if [ "$script_type" == "down" ]; then

  /sbin/ifconfig $dev down
  /usr/sbin/brctl delif $IF_NAME $dev

  exit 0
fi

if [ "$script_type" == "client-connect" ] && [ -n "$1" ]; then

  if [ -n "$username" ]; then
    if find_userpass_ip; then
      echo "ifconfig-push $IP $IP_MASK" > "$1"
    else
      myexit 1 "did not find username, probably internal error"
    fi
  else
    find_next_unused_ip
    echo "ifconfig-push $IP $IP_MASK" > "$1"
    echo "$IP" > "$LASTIP_FNAME"
  fi

  exit 0
fi

if [ "$script_type" == "client-disconnect" ]; then
  exit 0
fi

if [ "$script_type" == "user-pass-verify" ]; then

  if [ -n "$username" ]; then
    if [[ ! $username =~ ^[-a-zA-Z0-9\._:]+$ ]] || [[ ! $password =~ ^[-a-zA-Z0-9\._:]+$ ]]; then myexit 1 "bad characters in username or password"; fi
    if find_userpass_ip "checkpass"; then
      exit 0
    else
      find_next_unused_ip
      echo "CLIENT_LIST,$username,$password,$IP" >> "$CLIENTS_FNAME"
      echo "$IP" > "$LASTIP_FNAME"
    fi
  fi

  exit 0
fi

exit 1
